#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>

using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::FFAdj,  face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>, 
    std::vector<MyFace> , std::vector<MyEdge>  > {};
int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  
  int borderEdgeNum=0;
  // for(int i=0;i<m.FN();++i) // NO BUONO
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
  tri::Allocator<MyMesh>::CompactEveryVector(m);
  for(MyMesh::FaceIterator fi=m.face.begin(); fi!=m.face.end();++fi) // BUONO
      for(int j=0;j<3;++j)
      {
        if(!fi->IsV() && face::IsBorder(*fi,j))
        {
          int loopLen=0;
          face::Pos<MyFace> startPos(&*fi,j);
          face::Pos<MyFace> curPos=startPos;
          assert(startPos.IsBorder());
          do
          {
            do
            {
              curPos.F()->SetV();
              curPos.FlipE();
              curPos.FlipF();
            } while(!curPos.IsBorder());
            curPos.FlipV();
            ++loopLen;
          }
          while(curPos!=startPos);          
          printf("Found a loop of %i edge\n",loopLen);
        }
      }
  
  printf("Border Edge Number %i\n",borderEdgeNum);
  return 0;
}
